package com.fjhx.sale.service.contract;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import com.fjhx.item.entity.product.dto.ProductInfoSelectDto;
import com.fjhx.sale.entity.contract.dto.ContractDto;
import com.fjhx.sale.entity.contract.dto.ContractSelectDto;
import com.fjhx.sale.entity.contract.dto.DecisionAidDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.vo.ContractBudgetVo;
import com.fjhx.sale.entity.contract.vo.ContractPdfInfoVo;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.contract.vo.DecisionAidVo;
import com.fjhx.sale.entity.sale.vo.SaleQuotationVo;
import com.ruoyi.common.core.service.BaseService;

import java.util.List;
import java.util.Map;


/**
 * <p>
 * 外销合同表 服务类
 * </p>
 *
 * @author
 * @since 2023-04-17
 */
public interface ContractService extends BaseService<Contract> {

    /**
     * 外销合同表分页
     */
    Page<ContractVo> getPage(ContractSelectDto dto);

    /**
     * 利润预算分页
     */
    Page<ContractBudgetVo> getProfitBudgetPage(ContractSelectDto dto);

    /**
     * 利润结算分页
     */
    Page<ContractBudgetVo> getProfitClearingPage(ContractSelectDto dto);

    /**
     * 外销合同表明细
     */
    ContractVo detail(Long id);

    /**
     * 外销合同表新增
     */
    void add(ContractDto contractDto);

    /**
     * 外销合同表编辑
     */
    void edit(ContractDto contractDto);

    /**
     * 外销合同表删除
     */
    void delete(Long id);

    /**
     * 根据客户ID查询未包装的订单
     * @param customerId
     * @return
     */
    List<ContractVo> getNoPackContractByCustomerId(String customerId);

    /**
     * 根据合同ID集合查询合同
     */
    List<Contract> getByIds(List<Long> ids);

    /**
     * 获取合同明细
     */
    ContractPdfInfoVo getContractPdfInfo(Long id);

    /**
     * 查询销售额（合同总金额）
     * @param id（买方公司ID）
     */
    ContractVo  getSalesTotal(Long id);

    /**
     * 查询指定客户的每月合同总额
     * @param dto
     */
    List<ContractVo> getAmount(CustomerDto dto);

    /**
     * 查询成交单统计（合同）
     * @param query
     * @return
     */
    List<SaleQuotationVo> contractStatistics(QueryWrapper<Object> query);

    /**
     * 销售趋势（数据看板-产品分析页面）
     * @param productInfoDto
     */
    List<Map<String, Object>> saleTrend(ProductInfoSelectDto productInfoDto);

    /**
     * 销售统计(数据看板-销售分析页面开发)
     */
    Map<String, Object>  salesStatistics(ContractDto dto);

    /**
     * 国家销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> countrySalesStatistics(ContractDto dto);

    /**
     * 客户销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> customSalesStatistics(ContractDto dto);

    /**
     * 业务员销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> salesmanSalesStatistics(ContractDto dto);

    /**
     * 产品销售统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> productSalesStatistics(ContractDto dto);


    /**
     * 产品销售量统计(数据看板-销售分析页面开发)
     */
    List<ContractDto> salesVolumeStatistics(ContractDto dto);

    /**
     * 合同交接单
     */
    void contractHandover(ContractDto dto);

    /**
     * 合同销售统计（合同列表）
     */
    Map<String, Object> covenantStatistics(ContractDto dto);

    /**
     * 销售合同-》到账完成
     */
    void toTheAccount(Contract dto);

    /**
     * 决策辅助
     */
    DecisionAidVo decisionAid(DecisionAidDto dto);

}
